PAGE 60,132
TITLE CHKFSER.COM  Verify Volume Serial Number on Floppy diskette
; ---------------------------------------------------------------------

;  Syntax      CHKFSER d:xxxx-xxxx

;              (c) 2001   Ben Ritchey Jr., et al
;                         Propriety use only - All rights reserved

               .8086           ; Base Architecture
CSEG           SEGMENT
               ASSUME  cs:CSEG, ds:CSEG, es:CSEG, ss:CSEG

; Interrupts (ah=Func)
BIOSVID        EQU     10h     ; BIOS Video Int
BIOSEQU        EQU     11h     ;  "   Equipment Flags
BIOSMEM        EQU     12h     ;  "   Memory Size
BIOSDSK        EQU     13h     ;  "   Floppy Disk
BIOSCOM        EQU     14h     ;  "   Communications
BIOSEXT        EQU     15h     ;  "   Extended
BIOSKBD        EQU     16h     ;  "   Keyboard
BIOSPRT        EQU     17h     ;  "   Printer
BIOSRTC        EQU     1Ah     ;  "   Real Time Clock

DOS            EQU     21h     ; DOS Sys Func Req
ALMINT         EQU     4Ah     ; SysRQ User Alarm

; ---------------------------------------------------------------------
; Program Segment Prefix (PSP x 256/100h)
               org     0
PSPDOS         Label   Word    ; DOS Function for Terminate
               org     2
PSPFree        Label   Word    ; Seg of 1st Free/Pgm Para Size
               org     6h
PSPDOSFar      Label   Byte    ; OpCode Far Call DOS Int 21h
               org     0Ah
PSPTerm        Label   DWord   ; IP,CS - Terminate
               org     0Eh
PSPBreak       Label   DWord   ; IP,CS - Ctrl+Break
               org     12h
PSPError       Label   DWord   ; IP,CS - Critical Errors
               org     2Ch
PSPEnv         Label   Word    ; Seg of Environ (Pgm Copy)
               org     5Ch
PSPFCB1        Label   Byte    ; Formatted FCB/FSpec of Arg 1 x16
               org     6Ch
PSPFCB2        Label   Byte    ; Formatted FCB/FSpec of Arg 2 x20

               org     80h     ; Default Disk Trfr Area x128
PSPVLI         Label   Byte    ; Args Len (1st = Sp @ 81h if >0)
               org     82h
PSPArgs        Label   Byte    ; 'args...' if Vli >1

               PAGE
; ---------------------------------------------------------------------
               org     100h
Start          Label   Proc

               mov     RUNES,es
               mov     RUNSP,sp
               mov     ax,6200h  ; Get PSP Seg
               int     DOS

               mov     PSPES,bx  ; 1st Seg/Page
               mov     es,bx

               mov     ax,RUNES  ; Calculate ACTUAL Size  <==
               mov     bx,RUNSP  ; Convert Stack Ptr to Paras
               mov     cl,4
               shr     bx,cl   ;   to get Data + Stack (/16)
               inc     bx
               add     ax,bx   ; Add Stack Pages to DS (End of Pgm)
               mov     bx,es   ; Get Start of Program (PSP)
               sub     ax,bx   ; Subtract Start from End
               inc     ax      ; +1 for Ctl Blk
               mov     bx,ax   ; Now actual Seg/Page Size

;               mov     bx,PBPAGES  ; (to Plug Max No. Pages)
               xor     al,al
               mov     ah,4Ah  ; Re-Allocate
               int     DOS
               jnc     @F
               mov     RC,al
               jmp     EXIT

@@:            xor     ch,ch
               mov     cl,PSPVLI  ; Arg Len
               cmp     cl,12    ; Bad Args?
               je      @F
               jmp     ERROR

@@:            mov     bx,Offset PSPArgs+10
               dec     cx

; Validate 0-9, A-F (Cv a-f)
VSNO:          cmp     cx,2    
               jne     @F
               cmp     Byte Ptr [bx],':'
               je      NXSNO
               jmp     ERROR

@@:            cmp     cx,7
               jne     @F
               cmp     Byte Ptr [bx],'-'
               je      NXSNO
               jmp     ERROR

@@:            cmp     Byte Ptr [bx],48  ; 0?
               jge     @F
               jmp     ERROR
@@:            cmp     Byte Ptr [bx],102  ; f?
               jle     @F
               jmp     ERROR
@@:            cmp     Byte Ptr [bx],97  ; a?
               jl      @F

               sub     Byte Ptr [bx],32  ; Cv to Upper case
@@:            cmp     Byte Ptr [bx],70  ; F?
               jle     @F
               jmp     ERROR
@@:            cmp     Byte Ptr [bx],57  ; 9?
               jle     NXSNO
               cmp     Byte Ptr [bx],65  ; A?
               jge     @F
               jmp     ERROR

@@:            sub     Byte Ptr [bx],7  ; A after 9

NXSNO:         sub     Byte Ptr [bx],48  ; Binary
               dec     bx
               loop    VSNO

               mov     cl,4

               mov     al,PSPArgs+2  ; Cv to Packed
               shl     al,cl
               mov     ah,al
               mov     al,PSPArgs+3
               and     al,0Fh
               or      al,ah
               mov     BSNO+3,al

               mov     al,PSPArgs+4
               shl     al,cl
               mov     ah,al
               mov     al,PSPArgs+5
               and     al,0Fh
               or      al,ah
               mov     BSNO+2,al

               mov     al,PSPArgs+7
               shl     al,cl
               mov     ah,al
               mov     al,PSPArgs+8
               and     al,0Fh
               or      al,ah
               mov     BSNO+1,al

               mov     al,PSPArgs+9
               shl     al,cl
               mov     ah,al
               mov     al,PSPArgs+10
               and     al,0Fh
               or      al,ah
               mov     BSNO,al

; Read Diskette Boot Sector (0)
               mov     ax,3600h  ; Dummy access
               mov     dl,PSPArgs  ; Drive (A=1, B=2)
               sub     dl,9
               int     DOS

               mov     dl,PSPArgs  ; Drive (A=0, B=1)
               sub     dl,10
               cmp     dl,1
               jle     @F
               jmp     ERROR   ; Syntax

@@:            xor     dh,dh   ; Side/Head 0,1
               mov     cx,1    ; ch:Track 0, cl:Sector 1 (Boot Rec)
               mov     al,1    ; # Secs
               mov     bx,Offset BUFFER  ; es:bx = DTA
               mov     ah,2    ; Read Sector
               int     BIOSDSK
               jnc     @F      ; Err?
               mov     RC,ah
               jmp     EXIT
@@:            cmp     al,1    ; Read 1 Sec?
               je      @F
               mov     RC,109
               jmp     EXIT

@@:            mov     es,RUNES
               mov     si,Offset BUFFER+38  ; Flag?
               mov     al,[si]
               cmp     al,29h
               je      @F
               jmp     ERROR3

@@:            inc     si      ; Check Serial No.'s
               mov     di,si
               mov     si,Offset BSNO
               mov     cx,2
               cld
               repe    cmpsw

               jnz     ERROR2  ; *** BAD ***

               mov     ax,4C00h  ; Return/Exit
               int     DOS
               ret

; Syntax Error
ERROR:         mov     bx,1    ; Device STDOUT
               mov     cx,EMSGL  ; VLI
               mov     dx,Offset EMSG  ; Addr of Text
               mov     RC,102
               jmp     ERRXIT

; Serial mis-match
ERROR2:        mov     bx,1
               mov     cx,EMSG2L
               mov     dx,Offset EMSG2
               mov     RC,115
               jmp     ERRXIT

; Serial not set
ERROR3:        mov     bx,1
               mov     cx,EMSG3L
               mov     dx,Offset EMSG3
               mov     RC,116

ERRXIT:        mov     ax,4000h  ; Display [dx] for cx
               int     DOS     ; DOS

EXIT:          mov     al,RC   ; ErrorLevel
               mov     ah,4Ch  ; Return/Exit
               int     DOS
               ret
 
; ---------------------------------------------------------------------
; Data & Constants
RC             DB      0
RUNES          DW      0
PSPES          DW      0
RUNSP          DW      0
BSNO           DB      4 DUP(0)

EMSG           DB      16,' Error - Syntax is [ CHKFSER d:xxxx-xxxx ]'
EMSGL          EQU     $-EMSG
EMSG2          DB      16,' Error - Diskette SERIAL is invalid'
EMSG2L         EQU     $-EMSG2
EMSG3          DB      16,' Error - Diskette not serialized'
EMSG3L         EQU     $-EMSG3

BUFFER         DB      4096 DUP(0)  ; BOOT Sector data
; BUFFER+38 = Serial # FLAG (0=None)
; BUFFER+39 = Serial # START (4 bytes, L/Msb)

CSEG           ENDS
               END     Start
;*** End of Program ***

